@@ -53,7 +53,15 @@ CORS_OPTS = [
 ]
 
 
-class CORS(base.Middleware):
+class InvalidOriginError(Exception):
+    """Exception raised when Origin is invalid."""
+    def __init__(self, origin):
+        self.origin = origin
+        super(InvalidOriginError, self).__init__(
+            'CORS request from origin \'%s\' not permitted.' % origin)
+
+
+class CORS(base.ConfigurableMiddleware):
     """CORS Middleware.
 
     This middleware allows a WSGI app to serve CORS headers for multiple
@@ -63,6 +71,8 @@ class CORS(base.Middleware):
     """
 
     simple_headers = [
+        'Accept',
+        'Accept-Language',
         'Content-Type',
         'Cache-Control',
         'Content-Language',
@@ -71,14 +81,14 @@ class CORS(base.Middleware):
         'Pragma'
     ]
 
-    def __init__(self, application, conf=None):
-        super(CORS, self).__init__(application, conf)
+    def __init__(self, application, *args, **kwargs):
+        super(CORS, self).__init__(application, *args, **kwargs)
         # Begin constructing our configuration hash.
         self.allowed_origins = {}
         self._init_conf()
 
     @classmethod
-    def factory(cls, global_conf, allowed_origin, **local_conf):
+    def factory(cls, global_conf, **local_conf):
         """factory method for paste.deploy
 
         allowed_origin: Protocol, host, and port for the allowed origin.
@@ -88,11 +98,11 @@ class CORS(base.Middleware):
         allow_methods: List of HTTP methods to permit.
         allow_headers: List of HTTP headers to permit from the client.
         """
-
-        # Ensures allowed_origin config exists
-        return super(CORS, cls).factory(global_conf,
-                                        allowed_origin=allowed_origin,
-                                        **local_conf)
+        if ('allowed_origin' not in local_conf
+                and 'oslo_config_project' not in local_conf):
+            raise TypeError("allowed_origin or oslo_config_project "
+                            "is required")
+        return super(CORS, cls).factory(global_conf, **local_conf)
 
     def _init_conf(self):
         '''Initialize this middleware from an oslo.config instance.'''
@@ -261,15 +271,11 @@ class CORS(base.Middleware):
             return response
 
         # Is this origin registered? (Section 6.2.2)
-        origin = request.headers['Origin']
-        if origin not in self.allowed_origins:
-            if '*' in self.allowed_origins:
-                origin = '*'
-            else:
-                LOG.debug('CORS request from origin \'%s\' not permitted.'
-                          % (origin,))
-                return response
-        cors_config = self.allowed_origins[origin]
+        try:
+            origin, cors_config = self._get_cors_config_by_origin(
+                request.headers['Origin'])
+        except InvalidOriginError:
+            return response
 
         # If there's no request method, exit. (Section 6.2.3)
         if 'Access-Control-Request-Method' not in request.headers:
@@ -333,6 +339,16 @@ class CORS(base.Middleware):
 
         return response
 
+    def _get_cors_config_by_origin(self, origin):
+        if origin not in self.allowed_origins:
+            if '*' in self.allowed_origins:
+                origin = '*'
+            else:
+                LOG.debug('CORS request from origin \'%s\' not permitted.'
+                          % origin)
+                raise InvalidOriginError(origin)
+        return origin, self.allowed_origins[origin]
+
     def _apply_cors_request_headers(self, request, response):
         """Handle Basic CORS Request (Section 6.1)
 
@@ -345,12 +361,11 @@ class CORS(base.Middleware):
             return
 
         # Is this origin registered? (Section 6.1.2)
-        origin = request.headers['Origin']
-        if origin not in self.allowed_origins:
-            LOG.debug('CORS request from origin \'%s\' not permitted.'
-                      % (origin,))
+        try:
+            origin, cors_config = self._get_cors_config_by_origin(
+                request.headers['Origin'])
+        except InvalidOriginError:
             return
-        cors_config = self.allowed_origins[origin]
 
         # Set the default origin permission headers. (Sections 6.1.3 & 6.4)
         response.headers['Vary'] = 'Origin'
