@@ -28,12 +28,19 @@ class HealthcheckTests(test_base.BaseTestCase):
     def application(req):
         return 'Hello, World!!!'
 
+    def _do_test_request(self, conf={}, path='/healthcheck',
+                         accept='text/plain', method='GET'):
+        self.app = healthcheck.Healthcheck(self.application, conf)
+        req = webob.Request.blank(path, accept=accept, method=method)
+        res = req.get_response(self.app)
+        return res
+
     def _do_test(self, conf={}, path='/healthcheck',
                  expected_code=webob.exc.HTTPOk.code,
-                 expected_body=b''):
-        self.app = healthcheck.Healthcheck(self.application, conf)
-        req = webob.Request.blank(path)
-        res = req.get_response(self.app)
+                 expected_body=b'', accept='text/plain',
+                 method='GET'):
+        res = self._do_test_request(conf=conf, path=path,
+                                    accept=accept, method=method)
         self.assertEqual(expected_code, res.status_int)
         self.assertEqual(expected_body, res.body)
 
@@ -51,14 +58,17 @@ class HealthcheckTests(test_base.BaseTestCase):
         conf = {'path': '/hidden_healthcheck'}
         self._do_test(conf, path='/toto', expected_body=b'Hello, World!!!')
 
-    @mock.patch('logging.warn')
-    def test_disablefile_unconfigured(self, fake_warn):
+    @mock.patch('oslo_middleware.healthcheck.disable_by_file.LOG')
+    def test_disablefile_unconfigured(self, fake_log):
+        fake_warn = fake_log.warning
         conf = {'backends': 'disable_by_file'}
         self._do_test(conf, expected_body=b'OK')
         self.assertIn('disable_by_file', self.app._backends.names())
-        fake_warn.assert_called_once('DisableByFile healthcheck middleware '
-                                     'enabled without disable_by_file_path '
-                                     'set')
+        fake_warn.assert_called_once_with(
+            'DisableByFile healthcheck middleware '
+            'enabled without disable_by_file_path '
+            'set'
+        )
 
     def test_disablefile_enabled(self):
         conf = {'backends': 'disable_by_file',
@@ -66,6 +76,20 @@ class HealthcheckTests(test_base.BaseTestCase):
         self._do_test(conf, expected_body=b'OK')
         self.assertIn('disable_by_file', self.app._backends.names())
 
+    def test_disablefile_enabled_head(self):
+        conf = {'backends': 'disable_by_file',
+                'disable_by_file_path': '/foobar'}
+        self._do_test(conf, expected_body=b'', method='HEAD',
+                      expected_code=webob.exc.HTTPNoContent.code)
+
+    def test_disablefile_enabled_html_detailed(self):
+        conf = {'backends': 'disable_by_file',
+                'disable_by_file_path': '/foobar', 'detailed': True}
+        res = self._do_test_request(conf, accept="text/html")
+        self.assertIn(b'Result of 1 checks:', res.body)
+        self.assertIn(b'<TD>OK</TD>', res.body)
+        self.assertEqual(webob.exc.HTTPOk.code, res.status_int)
+
     def test_disablefile_disabled(self):
         filename = self.create_tempfiles([('test', 'foobar')])[0]
         conf = {'backends': 'disable_by_file',
@@ -75,6 +99,24 @@ class HealthcheckTests(test_base.BaseTestCase):
                       expected_body=b'DISABLED BY FILE')
         self.assertIn('disable_by_file', self.app._backends.names())
 
+    def test_disablefile_disabled_head(self):
+        filename = self.create_tempfiles([('test', 'foobar')])[0]
+        conf = {'backends': 'disable_by_file',
+                'disable_by_file_path': filename}
+        self._do_test(conf,
+                      expected_code=webob.exc.HTTPServiceUnavailable.code,
+                      expected_body=b'', method='HEAD')
+        self.assertIn('disable_by_file', self.app._backends.names())
+
+    def test_disablefile_disabled_html_detailed(self):
+        filename = self.create_tempfiles([('test', 'foobar')])[0]
+        conf = {'backends': 'disable_by_file',
+                'disable_by_file_path': filename, 'detailed': True}
+        res = self._do_test_request(conf, accept="text/html")
+        self.assertIn(b'<TD>DISABLED BY FILE</TD>', res.body)
+        self.assertEqual(webob.exc.HTTPServiceUnavailable.code,
+                         res.status_int)
+
     def test_two_backends(self):
         filename = self.create_tempfiles([('test', 'foobar')])[0]
         conf = {'backends': 'disable_by_file,disable_by_file',
